#include <boost/spirit/include/qi.hpp>
// allows one to incorporate semantic actions in the grammar as part of the
// production rules declarations (see below) in a functional programming like
// style
#include <boost/spirit/include/phoenix.hpp>

#include <string>
#include <iostream>


template <typename Iterator>
        struct arith : boost::spirit::qi::grammar<Iterator, double(), boost::spirit::ascii::space_type>
        {
            arith() : arith::base_type(expression)
            {
                using boost::spirit::qi::_1;
                using boost::spirit::qi::_val;

                using boost::spirit::qi::real_parser;
                using boost::spirit::qi::real_policies;
                real_parser<double, real_policies<double> > real;


                expression =
                    term                    [_val =  _1]    // the semantic action (given inside the braces [])
                    >> *(   ('+' >> term    [_val += _1])   // if the parser successfully matches the right hand
                        |   ('-' >> term    [_val -= _1])   // side of the production rule
                        )
                    ;

                term =
                    real             [_val =  _1]
                    ;
                // TODO: implement the rest of the grammar so that it can
                // handle more complex arithmetic expressions
            }
            boost::spirit::qi::rule<Iterator, double(), boost::spirit::ascii::space_type> term, expression;


        };


template <typename Iterator>
bool parse(Iterator &iter,
           Iterator end,
           const arith<Iterator> &g,
           double &result)
{
    return boost::spirit::qi::phrase_parse(iter, end, g, boost::spirit::ascii::space, result);
}

int main()
{
    std::string s = "2+4+5";
    std::string::const_iterator b = s.begin();
    std::string::const_iterator e = s.end();
    arith<std::string::const_iterator> arith_parser;
    double result;
    bool success = parse(b, e, arith_parser, result);
    if (success)
    {
        std::cout << "Result = " << result << std::endl;
    }
}
